@@ -164,8 +164,7 @@ ext2fs_dinode_lookup(EXT2FS_INFO *ext2fs, INUM_T inum)
     /*
      * Sanity check.
      */
-    if ((inum < fs->first_inum) || 
-	  (inum > getu32(fs, ext2fs->fs->s_inodes_count)) )
+    if ((inum < fs->first_inum) || (inum > fs->last_inum))
 		error("invalid inode number: %lu", (ULONG) inum);
 
     /*
@@ -1010,7 +1009,8 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 
 
 static void
-ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
+ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
+  int32_t sec_skew)
 {
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     FS_INODE *fs_inode;
@@ -1066,7 +1066,31 @@ ext2fs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
     fprintf(hFile, "size: %lu\n", (ULONG) fs_inode->size);
     fprintf(hFile, "num of links: %lu\n", (ULONG) fs_inode->nlink);
 
-    fprintf(hFile, "\nInode Times:\n");
+    if (sec_skew != 0) {
+        fprintf(hFile, "\nAdjusted Inode Times:\n");                  
+        fs_inode->mtime -= sec_skew;
+        fs_inode->atime -= sec_skew;
+        fs_inode->ctime -= sec_skew;
+   
+        fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+        fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
+        fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
+
+		if (fs_inode->dtime) {
+        	fs_inode->dtime -= sec_skew;
+			fprintf(hFile, "Deleted:\t%s", ctime(&fs_inode->dtime));
+        	fs_inode->dtime += sec_skew;
+		}
+
+        fs_inode->mtime += sec_skew;
+        fs_inode->atime += sec_skew;
+        fs_inode->ctime += sec_skew;
+
+        fprintf(hFile, "\nOriginal Inode Times:\n");             
+    }
+    else
+        fprintf(hFile, "\nInode Times:\n");
+
 	fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
 	fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
 	fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
