@@ -1162,7 +1162,8 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 
 
 static void
-fatfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
+fatfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
+  int32_t sec_skew)
 {
     FS_INODE *fs_inode;
 	FS_NAME *fs_name;
@@ -1184,7 +1185,25 @@ fatfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
 		fprintf(hFile, "Name: %s\n", fs_name->name);
 	}
 
-    fprintf(hFile, "\nDirectory Entry Times:\n");
+    if (sec_skew != 0) {
+        fprintf(hFile, "\nAdjusted Directory Entry Times:\n");                  
+        fs_inode->mtime -= sec_skew;
+        fs_inode->atime -= sec_skew;
+        fs_inode->ctime -= sec_skew;
+   
+		fprintf(hFile, "Written:\t%s", ctime(&fs_inode->mtime));
+		fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+		fprintf(hFile, "Created:\t%s", ctime(&fs_inode->ctime));
+
+        fs_inode->mtime += sec_skew;
+        fs_inode->atime += sec_skew;
+        fs_inode->ctime += sec_skew;
+
+        fprintf(hFile, "\nOriginal Directory Entry Times:\n");             
+    }
+    else
+    	fprintf(hFile, "\nDirectory Entry Times:\n");
+
     fprintf(hFile, "Written:\t%s", ctime(&fs_inode->mtime));
     fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
     fprintf(hFile, "Created:\t%s", ctime(&fs_inode->ctime));
