@@ -920,7 +920,8 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 
 
 static void
-ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
+ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock, 
+  int32_t sec_skew)
 {
 	FFS_INFO *ffs = (FFS_INFO *) fs;
 	FS_INODE *fs_inode;
@@ -948,7 +949,26 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
 	fprintf(hFile, "size: %lu\n", (ULONG) fs_inode->size);
 	fprintf(hFile, "num of links: %lu\n", (ULONG) fs_inode->nlink);
 
-	fprintf(hFile, "\nInode Times:\n");
+
+	if (sec_skew != 0) {
+		fprintf(hFile, "\nAdjusted Inode Times:\n");
+        fs_inode->mtime -= sec_skew;
+        fs_inode->atime -= sec_skew; 
+        fs_inode->ctime -= sec_skew;
+
+		fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+		fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
+		fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
+
+        fs_inode->mtime += sec_skew;
+        fs_inode->atime += sec_skew; 
+        fs_inode->ctime += sec_skew;
+
+		fprintf(hFile, "\nOriginal Inode Times:\n");
+	}
+	else
+		fprintf(hFile, "\nInode Times:\n");
+
 	fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
 	fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
 	fprintf(hFile, "Inode Modified:\t%s", ctime(&fs_inode->ctime));
