@@ -51,9 +51,12 @@
 
 #include "ntfs.h"
 
+/* Time skew of the system in seconds */
+static int32_t sec_skew = 0;
+
 
 void usage(char *myProg) {
-	printf("usage: %s [-adDFlpurV] [-f fstype] [-m dir/] [-z ZONE] image [inode]\n", 
+	printf("usage: %s [-adDFlpruV] [-f fstype] [-m dir/] [-z ZONE] [-s seconds] image [inode]\n", 
 	  myProg);
     printf("\tIf [inode] is not given, 2 is used\n");
 	printf("\t-a: Display \".\" and \"..\" entries\n");
@@ -64,10 +67,11 @@ void usage(char *myProg) {
 	printf("\t-m: Display output in mactime input format with\n");
 	printf("\t      dir/ as the actual mount point of the image\n");
 	printf("\t-p: Display full path for each file\n");
-	printf("\t-u: Display undeleted entries only\n");
 	printf("\t-r: Recurse on directory entries\n");
+	printf("\t-u: Display undeleted entries only\n");
 	printf("\t-V: Print version\n");
-	printf("\t-z: Time zone of original machine (i.e. EST or GMT) (only useful with -l)\n");
+	printf("\t-z: Time zone of original machine (i.e. EST5EDT or GMT) (only useful with -l)\n");
+	printf("\t-s seconds: Time skew of original machine (in seconds) (only useful with -l)\n");
     printf("\t-f fstype: Image file system type\n");
 	printf("Supported file system types:\n");
 	fs_print_types();
@@ -112,10 +116,27 @@ printit (FS_INFO *fs, FS_DENT *fs_dent, int flags, FS_DATA *fs_data)
 
 
 	if (localFlags & LCL_MAC) {
+		if (sec_skew != 0) {
+			fs_dent->fsi->mtime -= sec_skew;
+			fs_dent->fsi->atime -= sec_skew;
+			fs_dent->fsi->ctime -= sec_skew;
+		}
+
 		fs_dent_print_mac(stdout, fs_dent, flags, fs, fs_data, macpre);
+
+		if (sec_skew != 0) {
+			fs_dent->fsi->mtime += sec_skew;
+			fs_dent->fsi->atime += sec_skew;
+			fs_dent->fsi->ctime += sec_skew;
+		}
 	}
 
 	else if (localFlags & LCL_LONG) {
+		if (sec_skew != 0) {
+			fs_dent->fsi->mtime -= sec_skew;
+			fs_dent->fsi->atime -= sec_skew;
+			fs_dent->fsi->ctime -= sec_skew;
+		}
 
 		if (LCL_FULL & localFlags) 
 			fs_dent_print_long(stdout, fs_dent, flags, fs, fs_data);
@@ -126,6 +147,11 @@ printit (FS_INFO *fs, FS_DENT *fs_dent, int flags, FS_DATA *fs_data)
 			fs_dent->path = tmpptr;
 		}
 
+		if (sec_skew != 0) {
+			fs_dent->fsi->mtime += sec_skew;
+			fs_dent->fsi->atime += sec_skew;
+			fs_dent->fsi->ctime += sec_skew;
+		}
 	}
 	else {
 		if (LCL_FULL & localFlags) 
@@ -244,7 +270,7 @@ main(int argc, char **argv)
 
 	localFlags =  LCL_DIR | LCL_FILE;
 
-	while ((ch = getopt(argc, argv, "adDf:Fm:luprVz:")) > 0) {
+	while ((ch = getopt(argc, argv, "adDf:Fm:lprs:uVz:")) > 0) {
 		switch (ch) {
 		case '?':
 		default: 
@@ -276,12 +302,15 @@ main(int argc, char **argv)
 		case 'p':
 			localFlags |= LCL_FULL;
 			break;
-		case 'u':
-			flags &= ~FS_FLAG_UNALLOC;
-			break;
 		case 'r':
 			flags |= FS_FLAG_RECURSE;
 			break;
+		case 's':
+			sec_skew = atoi(optarg);
+			break;
+		case 'u':
+			flags &= ~FS_FLAG_UNALLOC;
+			break;
 		case 'V':
 			print_version();
 			exit(0);
