@@ -30,32 +30,44 @@
 FS_DATA *
 fs_data_alloc(u_int8_t type)
 {
-    FS_DATA *data = (FS_DATA *) mymalloc(sizeof(FS_DATA));
-    data->flags = FS_DATA_INUSE;
-    data->nsize = 128;
+	FS_DATA *data = (FS_DATA *) mymalloc(sizeof(FS_DATA));
+	data->nsize = 128;
 	data->name = (char *)mymalloc(data->nsize);
 
-    data->size = 0;
-    data->buflen = 0;
+	data->size = 0;
+	data->flags = 0;
+	data->run = NULL;
 	data->type = 0;
 	data->next = NULL;
 
 	if (type == FS_DATA_NONRES) {
-#define DEFAULT_RUN		8
-		int size = DEFAULT_RUN * sizeof(FS_DATA_RUN);
-		data->run = (FS_DATA_RUN *)mymalloc(size);
-		memset(data->run, 0, size);
-		data->flags |= FS_DATA_NONRES;
+    		data->buflen = 0;
+		data->buf = NULL;
+		data->flags = (FS_DATA_NONRES | FS_DATA_INUSE);
 	}
 	else if (type == FS_DATA_RES) {
 		data->buflen = 1024;
 		data->buf = (u_int8_t *)mymalloc(data->buflen);
-		data->flags |= FS_DATA_RES;
+		data->flags = (FS_DATA_RES | FS_DATA_INUSE);
+	}
+	else {
+		error ("fs_data_alloc: Invalid Type: %d\n", type);
 	}
 
 	return data;
 }
 
+
+FS_DATA_RUN *
+fs_data_run_alloc ()
+{
+	FS_DATA_RUN *fs_data_run = 
+	  (FS_DATA_RUN *)mymalloc(sizeof(FS_DATA_RUN));
+	memset (fs_data_run, 0, sizeof(FS_DATA_RUN));	
+	return fs_data_run;
+}
+
+
 /* Free a data_run list */
 void
 fs_data_free_run (FS_DATA_RUN *run)
@@ -100,12 +112,12 @@ void
 fs_data_clear_list(FS_DATA *data)
 {
 	while (data) {
-		data->flags = 0;
+		data->flags = data->size = data->type = data->id = 0;
 		if (data->run) {
 			fs_data_free_run(data->run);
 			data->run = NULL;
+			data->runlen = 0;
 		}
-
 		data = data->next;
 	}
 }
@@ -316,7 +328,7 @@ fs_data_put_run(FS_DATA *data_head,
 		 * to account for the cluster numbers we haven't seen yet
 		 */
 		if (start_vcn != 0) {
-			FS_DATA_RUN *fill = (FS_DATA_RUN *)mymalloc(sizeof(FS_DATA_RUN));
+			FS_DATA_RUN *fill = fs_data_run_alloc();
 			fill->flags = FS_DATA_FILLER;
 			fill->offset = 0;
 			fill->len = start_vcn; 
@@ -367,8 +379,7 @@ fs_data_put_run(FS_DATA *data_head,
 				 * the filler, so make a new start filler
 				 */
 				else {
-					FS_DATA_RUN *newfill = (FS_DATA_RUN *)
-					  mymalloc(sizeof(FS_DATA_RUN));
+					FS_DATA_RUN *newfill = fs_data_run_alloc();
 				
 					if (data_run_prev)
 						data_run_prev->next = newfill;
@@ -456,8 +467,7 @@ fs_data_put_run(FS_DATA *data_head,
 	}
 	/* we need to make a filler before it */
 	else  {
-		FS_DATA_RUN *tmprun;
-		tmprun = (FS_DATA_RUN *)mymalloc(sizeof(FS_DATA_RUN));
+		FS_DATA_RUN *tmprun = fs_data_run_alloc();
 		if (data_run_prev)
 			data_run_prev->next = tmprun;
 		else
