@@ -258,7 +258,7 @@ struct FS_INFO {
     void    (*file_walk) (FS_INFO *, FS_INODE *, u_int32_t, u_int16_t, int, 
 	  FS_FILE_WALK_FN, char *);
 	void	(*fsstat) (FS_INFO *, FILE *);
-	void	(*istat) (FS_INFO *, FILE *, INUM_T, int);
+	void	(*istat) (FS_INFO *, FILE *, INUM_T, int, int32_t);
     void    (*close) (FS_INFO *);
 };
 
@@ -531,6 +531,7 @@ extern OFF_T mylseek(int, OFF_T, int);
  */
 extern u_int8_t	guessu16(FS_INFO *, u_int8_t *, u_int16_t);
 extern u_int8_t	guessu32(FS_INFO *, u_int8_t *, u_int32_t);
+extern void putu16(FS_INFO *, u_int8_t *, u_int8_t *);
 
 /* 16-bit values */
 #define getu16(fs, x)   \
