@@ -63,3 +63,20 @@ guessu32(FS_INFO *fs, u_int8_t *x, u_int32_t val)
 
 	return 1;
 }
+
+
+/*
+ * Put a 16-bit value from y into x
+ */
+void 
+putu16 (FS_INFO *fs, u_int8_t *x, u_int8_t *y)
+{
+	if (fs->flags & FS_LIT_ENDIAN) {
+		x[0] = y[0];
+		x[1] = y[1];
+	}
+	else {
+		x[0] = y[1];
+		x[1] = y[0];
+	}
+}
