@@ -131,6 +131,11 @@ extern char *progname;
 
 static char *image;
 
+/* number of seconds time skew of system 
+ * if the system was 100 seconds fast, the value should be +100 
+ */
+static int32_t sec_skew = 0;
+
 /* atoinum - convert string to inode number */
 
 INUM_T  atoinum(const char *str)
@@ -150,13 +155,14 @@ INUM_T  atoinum(const char *str)
 
 static void usage()
 {
-    printf("usage: %s [-eomrvV] [-f fstype] device [inum... ]\n",
+    printf("usage: %s [-eomrvV] [-f fstype] [-s seconds] device [inum... ]\n",
 	  progname);
 
 	printf("\t-e: Display all inodes\n");
 	printf("\t-o: Display inodes that are removed, but sill open\n");
 	printf("\t-m: Display output in the mactime format (replaces ils2mac from TCT)\n");
 	printf("\t-r: Display removed inodes (default)\n");
+	printf("\t-s seconds: Time skew of original machine (in seconds)\n");
 	printf("\t-v: verbose output to stderr\n");
 	printf("\t-V: Display version number\n");
     printf("\t-f fstype: Image file system type\n");
@@ -228,15 +234,34 @@ static u_int8_t
 print_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 			        char *unused_context)
 {
+
+	if (sec_skew != 0) {
+		fs_inode->mtime -= sec_skew;
+		fs_inode->atime -= sec_skew;
+		fs_inode->ctime -= sec_skew;
+	}
     printf("%lu|%c|%d|%d|%lu|%lu|%lu",
 	   (ULONG) inum, (flags & FS_FLAG_ALLOC) ? 'a' : 'f',
 	   (int) fs_inode->uid, (int) fs_inode->gid,
 	   (ULONG) fs_inode->mtime, (ULONG) fs_inode->atime,
 	   (ULONG) fs_inode->ctime);
 
-	if (fs->flags & FS_HAVE_DTIME)
+	if (sec_skew != 0) {
+		fs_inode->mtime += sec_skew;
+		fs_inode->atime += sec_skew;
+		fs_inode->ctime += sec_skew;
+	}
+
+	if (fs->flags & FS_HAVE_DTIME) {
+		if (sec_skew != 0) 
+			fs_inode->dtime -= sec_skew;
+
     	printf("|%lu", (ULONG) fs_inode->dtime);
 
+		if (sec_skew != 0) 
+			fs_inode->dtime += sec_skew;
+	}
+
     printf("|%lo|%d|%lu|%lu|%lu\n",
 	   (ULONG) fs_inode->mode, (int) fs_inode->nlink,
 	   (ULONG) fs_inode->size, 
@@ -267,12 +292,24 @@ print_inode_mac(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 	/* Print the "ls" mode in ascii format */
 	make_ls(fs_inode->mode, ls, 12);
 
+	if (sec_skew != 0) {
+		fs_inode->mtime -= sec_skew;
+		fs_inode->atime -= sec_skew;
+		fs_inode->ctime -= sec_skew;
+	}
+
 	printf("%s|%d|%d|%d|0|%lu|%lu|%lu|%lu|%u|0\n", 
 	  ls,(int)fs_inode->nlink, (int)fs_inode->uid,
 	  (int)fs_inode->gid, (ULONG)fs_inode->size, 
 	  (ULONG)fs_inode->atime, (ULONG)fs_inode->mtime,
 	  (ULONG)fs_inode->ctime, (int)fs->block_size);
 
+	if (sec_skew != 0) {
+		fs_inode->mtime -= sec_skew;
+		fs_inode->atime -= sec_skew;
+		fs_inode->ctime -= sec_skew;
+	}
+
 	return WALK_CONT;
 }
 
@@ -299,7 +336,7 @@ int     main(int argc, char **argv)
      * Provide convenience options for the most commonly selected feature
      * combinations.
      */
-    while ((ch = getopt(argc, argv, "aAef:lLomrvVzZ")) > 0) {
+    while ((ch = getopt(argc, argv, "aAef:lLomrs:vVzZ")) > 0) {
 	switch (ch) {
 	default:
 	    usage();
@@ -319,6 +356,9 @@ int     main(int argc, char **argv)
 	case 'r':
 		argflags |= ARG_R;
 	    break;
+	case 's':
+		sec_skew = atoi(optarg);
+		break;
 	case 'v':
 	    verbose++;
 	    logfp = stderr;
