@@ -82,11 +82,12 @@ atoinum(const char *str)
 /* usage - explain and terminate */
 
 static void usage() {
-    printf("usage: %s [-b num] [-f fstype] [-z zone] [-vV] image inum\n", progname);
+    printf("usage: %s [-b num] [-f fstype] [-z zone] [-s seconds] [-vV] image inum\n", progname);
 	printf("\t-v: verbose output to stderr\n");
 	printf("\t-V: print version\n");
 	printf("\t-b num: force the display of NUM address of block pointers\n");
-	printf("\t-z zone: time zone of original machine (i.e. EST or GMT)\n");
+	printf("\t-z zone: time zone of original machine (i.e. EST5EDT or GMT)\n");
+	printf("\t-s seconds: Time skew of original machine (in seconds)\n");
     printf("\t-f fstype: Image file system type\n");
 	printf("Supported file system types:\n");
 	fs_print_types();
@@ -101,6 +102,7 @@ main(int argc, char **argv)
     int     ch;
     char   *fstype = DEF_FSTYPE;
 	FS_INFO		*fs;
+	int32_t	sec_skew = 0;
 
 	/* When > 0 this is the number of blocks to print, used for -b arg */
 	int numblock = 0; 
@@ -108,7 +110,7 @@ main(int argc, char **argv)
     progname = argv[0];
 
 
-    while ((ch = getopt(argc, argv, "b:f:vVz:")) > 0) {
+    while ((ch = getopt(argc, argv, "b:f:s:vVz:")) > 0) {
 	switch (ch) {
 	default:
 	    usage();
@@ -122,6 +124,9 @@ main(int argc, char **argv)
 	case 'f':
 	    fstype = optarg;
 	    break;
+	case 's':
+		sec_skew = atoi(optarg);
+		break;
 	case 'v':
 	    verbose++;
 	    logfp = stderr;
@@ -154,7 +159,7 @@ main(int argc, char **argv)
     fs = fs_open(argv[optind], fstype);
 	inum = atoinum(argv[++optind]);
 
-	fs->istat(fs, stdout, inum, numblock);
+	fs->istat(fs, stdout, inum, numblock, sec_skew);
 
     fs->close(fs);
     exit(0);
