@@ -36,6 +36,39 @@ ntfs_proc_attrlist(NTFS_INFO *, FS_INODE *, FS_DATA *);
 
 
 
+/* mini-design note:
+ * The MFT has entries for every file and dir in the fs.
+ * The first entry ($MFT) is for the MFT itself and it is used to find
+ * the location of the entire table because it can become fragmented.
+ * Therefore, the $Data attribute of $MFT is saved in the NTFS_INFO
+ * structure for easy access.  We also use the size of the MFT as
+ * a way to calculate the maximum MFT entry number (last_inum).
+ *
+ * Ok, that is simple, but getting the full $Data attribute can be tough
+ * because $MFT may not fit into one MFT entry (i.e. an attribute list). 
+ * We need to process the attribute list attribute to find out which
+ * other entries to process.  But, the attribute list attribute comes
+ * before any $Data attribute (so it could refer to an MFT that has not
+ * yet been 'defined').  Although, the $Data attribute seems to always 
+ * exist and define at least the run for the entry in the attribute list.
+ *
+ * So, the way this is solved is that generic mft_lookup is used to get
+ * any MFT entry, even $MFT.  If $MFT is not cached then we calculate 
+ * the address of where to read based on mutliplication and guessing.  
+ * When we are loading the $MFT, we set 'loading_the_MFT' to 1 so
+ * that we can update things as we go along.  When we read $MFT we
+ * read all the attributes and save info about the $Data one.  If
+ * there is an attribute list, we will have the location of the
+ * additional MFT in the cached $Data location, which will be 
+ * updated as we process the attribute list.  After each MFT
+ * entry that we process while loading the MFT, the 'final_inum'
+ * value is updated to reflect what we can currently load so 
+ * that the sanity checks still work.
+ */
+
+// set to 1 when $MFT is itself being loaded
+static u_int8_t loading_the_MFT = 0; 
+
 /**********************************************************************
  *
  *  MISC FUNCS
@@ -106,7 +139,7 @@ uni2ascii(char *uni, int ulen, char *asc, int alen)
 
 
 /*
- * Lookup up a given MFT (mftnum) in the MFT Table and save it in raw 
+ * Lookup up a given MFT entry (mftnum) in the MFT and save it in its
  * raw format in *mft.
  *
  * NOTE: This will remove the update sequence integrity checks in the
@@ -115,12 +148,14 @@ uni2ascii(char *uni, int ulen, char *asc, int alen)
 static void
 ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 {
-	DADDR_T	mftaddr;
+	DADDR_T	mftaddr, mftaddr2;
+	int mftaddr_len = 0;
 	OFF_T offset;
 	int i;
-	u_int8_t *origptr, *seqptr;		
 	FS_INFO *fs = (FS_INFO *)&ntfs->fs_info;
 	FS_DATA_RUN *data_run;
+	ntfs_upd *upd;
+	u_int16_t orig_seq;
 
 	/* sanity checks */
 	if (!mft)
@@ -131,46 +166,113 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 	if (mftnum > fs->last_inum)
 		error("inode number is too large (%lu)", (ULONG) mftnum);
 
-	/* The MFT may not be in consecutive clusters, so we need to use its
-	 * data attribute run list to find out what address to read
-	 *
-	 * This is why we cached it
+	/* If mft_data (the cached $Data attribute of $MFT) is not there yet, 
+	 * then we have not started to load $MFT yet.  In that case, we will
+	 * 'cheat' and calculate where it goes.  This should only be for
+	 * $MFT itself, in which case the calculation is easy
 	 */
-	mftaddr = 0;
-	data_run = ntfs->mft_data->run; 
+	if (!ntfs->mft_data) {
 
-	/* The byte offset within the data stream */
-	offset = mftnum * ntfs->mft_rsize_b;
+		/* This is just a random check with the assumption being that
+		 * we don't want to just do a guess calculation for a very large
+		 * MFT entry
+		 */
+		if (mftnum > NTFS_LAST_DEFAULT_INO)
+			error ("Error trying to load a high MFT entry when the MFT itself has not been loaded (%d)", mftnum);
 
-	/* NOTE: data_run values are in clusters */
-	while (data_run) {
- 		OFF_T run_len = data_run->len * ntfs->csize_b;
+		mftaddr = ntfs->root_mft_addr +  mftnum * ntfs->mft_rsize_b;
+		mftaddr2 = 0;
+	}
+	else {
+		/* The MFT may not be in consecutive clusters, so we need to use its
+		 * data attribute run list to find out what address to read
+		 *
+		 * This is why we cached it
+		 */
 
-		/* Is our MFT is in this run */
-		if (offset < run_len) {
-			mftaddr = data_run->offset * ntfs->csize_b + offset;
-			break;
+		// will be set to the address of the MFT entry
+		mftaddr = mftaddr2 = 0;  
+
+		/* The byte offset within the $Data stream */
+		offset = mftnum * ntfs->mft_rsize_b;
+
+		/* data_run values are in clusters 
+		 * cycle through the runs in $Data and identify which
+		 * one has the MFT entry that we want
+		 */
+		for (data_run = ntfs->mft_data->run; 
+		  data_run != NULL; data_run = data_run->next) {
+			/* The length of this run */
+			OFF_T run_len = data_run->len * ntfs->csize_b;
+
+			/* Is our MFT entry is in this run */
+			if (offset < run_len) {
+
+				/* special case where the MFT entry crosses
+				 * a run (only happens when cluster size is 512-bytes
+				 * and there are an odd number of clusters in the run)
+				 */
+				if (run_len < offset + ntfs->mft_rsize_b) {
+
+					if (data_run->next == NULL) 
+						error("MFT entry crosses a cluster and there are no more clusters!");
+
+					/* Assign address where the remainder of the entry is */
+					mftaddr2= data_run->next->offset * ntfs->csize_b;	
+
+					/* this should always be 512, but just in case */
+					mftaddr_len = run_len - offset;
+				}	
+
+				/* Assign address of where the MFT entry starts */
+				mftaddr = data_run->offset * ntfs->csize_b + offset;
+				break;
+			}
+
+			/* decrement the offset we are looking for */
+			offset -= run_len;
 		}
 
-		/* decrement the offset we are looking for */
-	  	offset -= run_len;
-		data_run = data_run->next;
+		/* Did we find it? */
+		if (!mftaddr)
+			error ("Error finding MFT entry %d in $MFT", mftnum);
 	}
 
-	if (!mftaddr)
-		error ("Error finding offset in MFT: %d", offset);
 
-	fs_read_random(&ntfs->fs_info, (char *)mft, ntfs->mft_rsize_b,
-	  mftaddr, "mft read");
+	/* can we do just one read or do we need multiple? */
+	if (mftaddr2) {
+		/* read the first part into mft */
+		fs_read_random(&ntfs->fs_info, (char *)mft, 
+		  mftaddr_len, mftaddr, "mft read");
+
+		/* read the second part into mft */
+		fs_read_random(&ntfs->fs_info, (char *)((int)mft + mftaddr_len), 
+		  ntfs->mft_rsize_b - mftaddr_len, mftaddr2, "mft read");
+	}
+	else {	
+		/* read the raw entry into mft */
+		fs_read_random(&ntfs->fs_info, (char *)mft, 
+		  ntfs->mft_rsize_b, mftaddr, "mft read");
+	}
+
+	/* if we are saving into the NTFS_INFO structure, assign mnum too */
+	if ((int)mft == (int)ntfs->mft)
+		ntfs->mnum = mftnum;
 
 	/* Sanity Check */
-	if (getu32(fs, mft->magic) != NTFS_MFT_MAGIC)
+	/* @@@ What else should we do with a BAAD entry ... */
+	if ((getu32(fs, mft->magic) != NTFS_MFT_MAGIC) &&
+	 (getu32(fs, mft->magic) != NTFS_MFT_MAGIC_BAAD))
 		error ("entry %d has an invalid MFT magic: %x", mftnum,
 		  getu32(fs, mft->magic));
 
-	/* 
-	 * replace the values that were used for the update sequence 
-	 * (integrity checks)
+	/* The MFT entries have error and integrity checks in them
+	 * called update sequences.  They must be checked and removed
+	 * so that later functions can process the data as normal. 
+	 * They are located in the last 2 bytes of each 512-byte sector
+	 *
+	 * We first verify that the the 2-byte value is a give value and
+	 * then replace it with what should be there
 	 */
 
 	/* sanity check so we don't run over in the next loop */
@@ -178,31 +280,33 @@ ntfs_mft_lookup(NTFS_INFO *ntfs, ntfs_mft *mft, u_int32_t mftnum)
 		error ("More Update Sequence Entries than MFT size");
 
 
-	if (ntfs->ver == NTFS_VINFO_XP)
-		origptr = &mft->upd.upd_31.upd_seq;
-	else
-		origptr = &mft->upd.upd_30.upd_seq;
+	/* Apply the update sequence structure template */
+	upd = (ntfs_upd *)((int)mft + getu16(fs, mft->upd_off));
+	
+	/* Get the sequence value that each 16-bit value should be */
+	orig_seq = getu16(fs, upd->upd_val);
 
+	/* cycle through each sector */
 	for (i = 1; i < getu16(fs, mft->upd_cnt); i++) {
+		/* The offset into the buffer of the value to analyze */
+		int offset = i * ntfs->ssize_b - 2;
 
-		seqptr = (u_int8_t *)((int)mft + i * ntfs->ssize_b - 2);		
+		/* get the current sequence value */
+		u_int16_t cur_seq = getu16(fs, (int)mft + offset);
 
-		if (ntfs->ver == NTFS_VINFO_XP) {
-			if (getu16(fs, seqptr) != getu16(fs, mft->upd.upd_31.upd_val))
-				error ("incorrect 3.1 update sequence value: MFT %d  Seq: %d %d",
-				  getu16(fs, mft->upd.upd_31.upd_val), getu16(fs, seqptr), ntfs->ver);
-		} 
-		else {
-			if (getu16(fs, seqptr) != getu16(fs, mft->upd.upd_30.upd_val))
-				error ("incorrect 3.0 update sequence value: MFT %d  Seq: %d %d",
-				  getu16(fs, mft->upd.upd_30.upd_val), getu16(fs, seqptr), ntfs->ver);
+		/* get the replacement value */
+		u_int16_t cur_repl = 
+			  getu16(fs, &upd->upd_seq + (i-1) * 2);
+
+		/* are they the same? */
+		if (cur_seq != orig_seq) {
+			error ("Incorrect update sequence value in MFT %d\nUpdate Value: 0x%x Actual Value: 0x%x Replacement Value: 0x%x\nThis is typically because of a corrupted entry",
+			  mftnum, orig_seq, cur_seq, cur_repl);
 		}
 
-		/* don't use getu16 because it needs to be stored in the same
-		 * order
-		 */
-		*seqptr++ = *origptr++;
-		*seqptr = *origptr++;
+		/* Save the replacement value in the sequence location */
+		putu16(fs, (u_int8_t *)(int)mft + offset, 
+			  (u_int8_t *)&cur_repl);
 	}
 
 	return;
@@ -276,9 +380,7 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 	while ((NTFS_RUNL_LENSZ(run) != 0) && (NTFS_RUNL_OFFSZ(run) != 0) ) {
 
 		/* allocate a new data_run */
-		data_run = (FS_DATA_RUN *)mymalloc(sizeof(FS_DATA_RUN));
-		data_run->flags = 0;
-		data_run->next = NULL;
+		data_run = fs_data_run_alloc();
 
 		/* make the list, unless its the first pass & then we set the head */
 		if (data_run_prev)
@@ -312,6 +414,15 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 				data_run->offset |= (int64_t)((int64_t)0xff << (i * 8) );
 		}
 
+		/* The NT 4.0 version of NTFS uses an offset of -1 to represent
+		 * a hole, so add the sparse flag and make it look like the 2K
+		 * version with a offset of 0
+		 */
+		if ((data_run->offset == -1) && (ntfs->ver == NTFS_VINFO_NT)) {
+			data_run->flags |= FS_DATA_SPARSE;
+			data_run->offset = 0;
+		}
+
 		/* A Sparse file has a run with an offset of 0
 		 * there is a special case though of the BOOT MFT entry which
 		 * is the super block and has a legit offset of 0.
@@ -322,7 +433,7 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 		 * For sparse files the next run will have its offset relative 
 		 * to the current "pre_off" so skip that code
 		 */
-		if ((data_run->offset) || (ntfs->mnum == NTFS_MFT_BOOT)) {
+		else if ((data_run->offset) || (ntfs->mnum == NTFS_MFT_BOOT)) {
 			data_run->offset += pre_off;
 			pre_off = data_run->offset;
 		}
@@ -337,6 +448,17 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
 
 	}
 
+	/* special case, only seen on the $BADCLUST mft entry, which 
+	 * always has a weird size.
+	 * Basically, we check if there is only one run entry and it
+	 * is sparse, then get rid of it
+	 */
+	if ((data_run_head != NULL) && (data_run_head->next == NULL) && 
+	  (data_run_head->flags & FS_DATA_SPARSE)) {
+		free(data_run_head);
+		data_run_head = NULL;
+	}
+
 	return data_run_head;
 }
 
@@ -487,12 +609,12 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
   int len)
 {
 	ntfs_attr 	*attr = attrseq;
-	FS_DATA_RUN *fs_data_run;
+	FS_DATA		*fs_data_attrl = NULL, *fs_data;
 	char		name[NTFS_MAXNAMLEN+1];
 	u_int64_t	runlen;
 	FS_INFO 	*fs = (FS_INFO *)&ntfs->fs_info;
 
-	/* Cycle through the list */
+	/* Cycle through the list of attributes */
 	for ( ; ((int)attr >= (int)attrseq) && 
 			((int)attr <= ((int)attrseq + len)) && 
 			(getu32(fs, attr->len) > 0 &&
@@ -510,22 +632,6 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		 */
 		if (attr->res == NTFS_MFT_RES) {
 
-			/* 
-			 * If this is a no-name $Data attribute, we will use it to get
-			 * the FS_INODE size value.  The size == 0 check is for when
-			 * processing this again because of an attribute list
-			 */
-			if ((type == NTFS_ATYPE_DATA) && (attr->nlen == 0) &&
-			  (fs_inode->size == 0))
-				fs_inode->size = getu32(fs, attr->c.r.ssize);
-
-			/* IF this a directory, we will use the size of the IDXALLOC
-			 * and IDXROOT attributes for the FS_INODE size
-			 */
-			else if ((type == NTFS_ATYPE_IDXROOT) || 
-			  (type == NTFS_ATYPE_IDXALLOC))
-				fs_inode->size += getu32(fs, attr->c.r.ssize);
-
 			/* Copy the name into ASCII */
 			if (attr->nlen)
 				uni2ascii(&attr->c.r.name, attr->nlen, name, sizeof(name));
@@ -547,21 +653,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		 * list
 		 */
 		else {
-
-			/* get the full size if we have the $Data attr */
-			if ((type == NTFS_ATYPE_DATA) && (attr->nlen == 0) &&
-			  (fs_inode->size == 0))
-				fs_inode->size = getu64(fs, attr->c.nr.ssize);
-
-			/* Sum the size of idxroot and idxalloc for directories */
-			else if ((type == NTFS_ATYPE_IDXROOT) || 
-			  (type == NTFS_ATYPE_IDXALLOC))
-				fs_inode->size += getu32(fs, attr->c.nr.ssize);
-
-			/* convert the run to generic form */
-			fs_data_run = ntfs_make_data_run(ntfs, 
-			  (ntfs_runlist *) ((int)attr + getu16(fs, attr->c.nr.run_off)),
-			  &runlen);
+			FS_DATA_RUN *fs_data_run;
 
 			/* Copy the name */
 			if (attr->nlen)
@@ -571,6 +663,11 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 			else
 				strncpy(name, "N/A", NTFS_MAXNAMLEN+1);
 
+			/* convert the run to generic form */
+			fs_data_run = ntfs_make_data_run(ntfs, 
+			  (ntfs_runlist *) ((int)attr + getu16(fs, attr->c.nr.run_off)),
+			  &runlen);
+
 			/* Add the run to the list */
 			fs_inode->attr = fs_data_put_run(fs_inode->attr, 
 			  getu64(fs, attr->c.nr.start_vcn), runlen, fs_data_run,
@@ -581,6 +678,7 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 
 		/* 
 		 * Special Cases, where we grab additional information
+		 * regardless if they are resident or not
 		 */
 
 		/* Standard Information (is always resident) */
@@ -644,18 +742,83 @@ ntfs_proc_attrseq (NTFS_INFO *ntfs, FS_INODE *fs_inode, ntfs_attr *attrseq,
 		}
 
 		/* If this is an attribute list than we need to process
-		 * it to get the list of other entries to read 
+		 * it to get the list of other entries to read.  But, because
+		 * of the wierd scenario of the $MFT having an attribute list
+		 * and not knowing where the other MFT entires are yet, we wait 
+		 * until the end of the attrseq to processes the list and then
+		 * we should have the $Data attribute loaded
 		 */
 		else if (type == NTFS_ATYPE_ATTRLIST) {
-			/* we will have to lookup the exact fs_data entry */
-			FS_DATA *fs_data = fs_data_lookup (fs_inode->attr, 
+			if (fs_data_attrl) {
+				printf ("Multiple instances of attribute lists in the same MFT\n");
+				error ("I didn't realize that could happen, contact the developers");
+			}
+			fs_data_attrl = fs_data_lookup (fs_inode->attr, 
 			  NTFS_ATYPE_ATTRLIST, getu16(fs, attr->id));
-
-			ntfs_proc_attrlist(ntfs, fs_inode, fs_data);
 		}
 
 	} /* end of for loop */
 
+
+
+	/* we recalc our size everytime through here.  It is not the most
+	 * effecient, but easiest with all of the processing of attribute
+	 * lists and such
+	 */
+	fs_inode->size = 0;
+	for (fs_data = fs_inode->attr; 
+	  fs_data != NULL;
+	  fs_data = fs_data->next) {
+
+		if ( (fs_data->flags & FS_DATA_INUSE) == 0)
+			continue;
+
+		/* we account for the size of $Data, and directory locations */
+		if ( (fs_data->type == NTFS_ATYPE_DATA) ||
+		  (fs_data->type == NTFS_ATYPE_IDXROOT) ||
+		  (fs_data->type == NTFS_ATYPE_IDXALLOC) )
+			fs_inode->size += fs_data->size;
+	}
+
+	/* Are we currently in the process of loading $MFT? */
+	if (loading_the_MFT == 1) {
+
+		/* If we don't even have a mini cached version, get it now 
+		 * Even if we are not done because of attribute lists, then we
+		 * should at least have the head of the list 
+		 */
+		if (!ntfs->mft_data)  {
+
+			for (fs_data = fs_inode->attr; 
+			  fs_data != NULL;
+		  	  fs_data = fs_data->next) {
+
+				if ( (fs_data->flags & FS_DATA_INUSE) &&
+			 	  (fs_data->type == NTFS_ATYPE_DATA) &&
+			 	  (strcmp (fs_data->name, "$Data") == 0) ) {
+					ntfs->mft_data = fs_data;
+					break;
+				}
+			}
+		}
+
+		/* Update the inode count based on the current size 
+		 * IF $MFT has an attribute list, this value will increase each
+		 * time
+		 */
+		fs->inum_count = ntfs->mft_data->size / ntfs->mft_rsize_b;
+		fs->last_inum = fs->inum_count - 1;
+	}
+
+	/* If there was an attribute list, process it now, we wait because
+	 * the list can contain MFT entries that are described in $Data
+	 * of this MFT entry.  For example, part of the $DATA attribute
+	 * could follow the ATTRLIST entry, so we read it first and then 
+	 * process the attribute list
+	 */
+	if (fs_data_attrl) 
+		ntfs_proc_attrlist(ntfs, fs_inode, fs_data_attrl);
+
 	return;
 }
 
@@ -705,7 +868,7 @@ buffer_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
     int len;
 
     if (!act_buf)
-        error("The Action Buffer is NULL (??)");
+        error("The Action Buffer is NULL (?)");
 
     len = ((size < act_bufleft) ? size : act_bufleft);
 
@@ -764,7 +927,6 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 	  (list) && ((DADDR_T)list < endaddr);
 	  list = (ntfs_attrlist *)((int)list + getu16(fs, list->len)) )
 	{
-		ntfs_attr *attr;
 		int mftnum;
 		u_int32_t type;
 		u_int16_t id, i;
@@ -785,7 +947,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 		if (hist[i] == mftnum) 
 			continue;
 	
-		/* THis is a new one, add it to the history, and process it */
+		/* This is a new one, add it to the history, and process it */
 		if (histcnt < 256)
 			hist[histcnt++] = mftnum;
 
@@ -801,24 +963,35 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 		/* Sanity check. */
 		if (mftnum < ntfs->fs_info.first_inum || 
 		  mftnum > ntfs->fs_info.last_inum) {
-			error("Invalid MFT file reference in attribute list: %lu", 
-			  (ULONG) mftnum);
+			error("Invalid MFT file reference (%lu) in the attribute list of MFT %lu", 
+			  (ULONG) mftnum, ntfs->mnum);
 		}
 
 		ntfs_mft_lookup(ntfs, mft, mftnum);
 
 		/* verify that this entry refers to the original one */
-		if (getu48(fs, mft->file_ref) != ntfs->mnum)
-			error ("Extension record %d is not for attribute list: %d %d", 
-			  mftnum, (int)getu48(fs, mft->file_ref), (int)ntfs->mnum);
+		if (getu48(fs, mft->file_ref) != ntfs->mnum) {
 
+			/* Before we raise alarms, check if the original was
+			 * unallocated.  If so, then the list entry could 
+			 * have been reallocated, so we will just ignore it
+			 */
+			if ((getu16(fs, ntfs->mft->flags) & NTFS_MFT_INUSE) == 0)
+				continue;
+			else
+				error ("Extension record %lu (file ref = %lu) is not for attribute list of %lu", 
+			  (ULONG)mftnum, 
+			  (ULONG)getu48(fs, mft->file_ref),
+			  (ULONG)ntfs->mnum);
+
+		}	
 		/* 
 		 * Process the attribute seq for this MFT entry and add them
 		 * to the FS_INODE structure
 		 */
-		attr = (ntfs_attr *) ((int)mft + (int)getu16(fs, mft->attr_off));
 
-		ntfs_proc_attrseq (ntfs, fs_inode, attr, 
+		ntfs_proc_attrseq (ntfs, fs_inode, 
+		  (ntfs_attr *) ((int)mft + (int)getu16(fs, mft->attr_off)),
 	  	  ntfs->mft_rsize_b - getu16(fs, mft->attr_off));
 	}
 
@@ -1005,32 +1178,6 @@ ntfs_attrname_lookup(FS_INFO *fs, u_int16_t type, char *name, int len)
 	snprintf(name, len, "?");
 }
 
-/* load the FS_INODE structure for the mft entry for the mft 
- * we will need this for every future mft entry lookup so cache it
- */
-static void
-ntfs_load_mft (NTFS_INFO *ntfs)
-{
-	FS_INFO *fs = (FS_INFO *)&ntfs->fs_info;
-
-	/* The MFT entry is number 0, which is at the location described
-	 * in the super block.  Therefore, we do not need to call the 
-	 * special load_mft function
-	 */
-	fs_read_random(fs, (char *)ntfs->mft, ntfs->mft_rsize_b,
-	  ntfs->root_mft_addr, "mft read");
-	ntfs->mnum = NTFS_MFT_MFT;
-
-	ntfs->mft_inode = fs_inode_alloc(NTFS_NDADDR, NTFS_NIADDR);
-	ntfs_copy_inode(ntfs, ntfs->mft_inode);
-
-	/* cache the data attribute */
-	ntfs->mft_data = fs_data_lookup(ntfs->mft_inode->attr, NTFS_ATYPE_DATA, 0);
-	if (!ntfs->mft_data)
-		error ("Data Attribute not found in MFT MFT");
-
-}
-
 
 /* Load the block bitmap $Data run */
 static void
@@ -1066,9 +1213,7 @@ ntfs_load_bmap(NTFS_INFO *ntfs)
 
 /*
  * Load the VOLUME MFT entry and the VINFO attribute so that we
- * can identify the volume version of this.  This is important because
- * the MFT layout is different for XP than NT and 2K.  We can not 
- * use inode_lookup because it relies on already knowing the version
+ * can identify the volume version of this.  
  */
 static void
 ntfs_load_ver (NTFS_INFO *ntfs)
@@ -1077,14 +1222,7 @@ ntfs_load_ver (NTFS_INFO *ntfs)
 	FS_INODE *fs_inode;
 	FS_DATA *fs_data;
 
-	/* The VOLUME entry is number 4 */
-	fs_read_random(fs, (char *)ntfs->mft, ntfs->mft_rsize_b,
-	  ntfs->root_mft_addr + NTFS_MFT_VOL * ntfs->mft_rsize_b, 
-	  "mft read");
-	ntfs->mnum = NTFS_MFT_VOL;
-
-	fs_inode = fs_inode_alloc(NTFS_NDADDR, NTFS_NIADDR);
-	ntfs_copy_inode(ntfs, fs_inode);
+	fs_inode = ntfs_inode_lookup(fs, NTFS_MFT_VOL);
 
 	/* cache the data attribute */
 	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_VINFO, 0);
@@ -1107,6 +1245,9 @@ ntfs_load_ver (NTFS_INFO *ntfs)
 	else {
 		error ("VINFO is a non-resident attribute");
 	}
+
+	fs_inode_free(fs_inode);
+	return;
 }
 
 
@@ -1414,7 +1555,8 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 
 
 static void
-ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
+ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock, 
+  int32_t sec_skew)
 {
     FS_INODE *fs_inode;
 	FS_DATA *fs_data;
@@ -1448,7 +1590,25 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
     }
 
 	/* Standard Info Times */
-	fprintf(hFile, "\n$STANDARD_INFORMATION Times:\n");
+    if (sec_skew != 0) {
+		fprintf(hFile, "\nAdjusted $STANDARD_INFORMATION Times:\n");
+        fs_inode->mtime -= sec_skew;
+        fs_inode->atime -= sec_skew;
+        fs_inode->ctime -= sec_skew;
+   
+    	fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
+    	fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
+    	fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
+
+        fs_inode->mtime += sec_skew;
+        fs_inode->atime += sec_skew;
+        fs_inode->ctime += sec_skew;
+
+		fprintf(hFile, "\nOriginal $STANDARD_INFORMATION Times:\n");
+    }
+    else
+		fprintf(hFile, "\n$STANDARD_INFORMATION Times:\n");
+
     fprintf(hFile, "File Modified:\t%s", ctime(&fs_inode->mtime));
     fprintf(hFile, "MFT Modified:\t%s", ctime(&fs_inode->ctime));
     fprintf(hFile, "Accessed:\t%s", ctime(&fs_inode->atime));
@@ -1457,30 +1617,51 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock)
 	fs_data = fs_data_lookup(fs_inode->attr, NTFS_ATYPE_FNAME, 0);
 	if(fs_data) {
         ntfs_attr_fname *fname = (ntfs_attr_fname *)fs_data->buf;
-		time_t time;
+		time_t cr_time, m_time, c_time, a_time;
 
-        fprintf(hFile, "\n$FILE_NAME Times:\n");
 
-		time = nt2unixtime(getu64(fs, fname->crtime));
-    	fprintf(hFile, "Created:\t%s", ctime(&time));
-    
-    
+		cr_time = nt2unixtime(getu64(fs, fname->crtime));
         /* altered - modified */
-		time = nt2unixtime(getu64(fs, fname->mtime));
-    	fprintf(hFile, "File Modified:\t%s", ctime(&time));
-
+		m_time = nt2unixtime(getu64(fs, fname->mtime));
         /* MFT modified */
-		time = nt2unixtime(getu64(fs, fname->ctime));
-    	fprintf(hFile, "MFT Modified:\t%s", ctime(&time));
-
+		c_time = nt2unixtime(getu64(fs, fname->ctime));
         /* Access */
-		time = nt2unixtime(getu64(fs, fname->atime));
-    	fprintf(hFile, "Accessed:\t%s", ctime(&time));
+		a_time = nt2unixtime(getu64(fs, fname->atime));
+
+    	if (sec_skew != 0) {
+        	fprintf(hFile, "\nAdjusted $FILE_NAME Times:\n");
+        	cr_time -= sec_skew;
+        	m_time -= sec_skew;
+        	a_time -= sec_skew;
+        	c_time -= sec_skew;
+
+			fprintf(hFile, "Created:\t%s", ctime(&cr_time));
+			fprintf(hFile, "File Modified:\t%s", ctime(&m_time));
+			fprintf(hFile, "MFT Modified:\t%s", ctime(&c_time));
+			fprintf(hFile, "Accessed:\t%s", ctime(&a_time));
+
+        	cr_time += sec_skew;
+        	m_time += sec_skew;
+        	a_time += sec_skew;
+        	c_time += sec_skew;
+
+        	fprintf(hFile, "\nOriginal $FILE_NAME Times:\n");
+		}
+		else
+        	fprintf(hFile, "\n$FILE_NAME Times:\n");
+
+
+    	fprintf(hFile, "Created:\t%s", ctime(&cr_time));
+    	fprintf(hFile, "File Modified:\t%s", ctime(&m_time));
+    	fprintf(hFile, "MFT Modified:\t%s", ctime(&c_time));
+    	fprintf(hFile, "Accessed:\t%s", ctime(&a_time));
+
 /*
-		printf ("fsize: %lu  Alloc size: %lu\n",
+        printf ("fsize: %lu  Alloc size: %lu\n",
 		  getu64(fs, fname->real_fsize),
 		  getu64(fs, fname->alloc_fsize));
-		  */
+*/
+
     }
 
     fs_data = fs_inode->attr;
@@ -1533,6 +1714,8 @@ ntfs_open(const char *name, unsigned char ftype)
 	FS_INFO *fs = &(ntfs->fs_info);
 	int len;
 
+	loading_the_MFT = 0;
+
 	if ((ftype & FSMASK) != NTFS_TYPE)
 		error ("Invalid FS type in ntfs_open");
 
@@ -1612,17 +1795,36 @@ ntfs_open(const char *name, unsigned char ftype)
 
 	fs->root_inum = NTFS_ROOTINO;
 	fs->first_inum = NTFS_FIRSTINO;
+	fs->last_inum = NTFS_LAST_DEFAULT_INO;
+	ntfs->mft_data = NULL;
 
-	/* load the data run for the MFT table */
-	ntfs_load_mft(ntfs);
 
-	/* load the version of the file system */
-	ntfs_load_ver(ntfs);
+	/* load the data run for the MFT table into ntfs->mft */
+	loading_the_MFT = 1;
+	ntfs_mft_lookup(ntfs, ntfs->mft, NTFS_MFT_MFT);
+
+    ntfs->mft_inode = fs_inode_alloc(NTFS_NDADDR, NTFS_NIADDR);
+    ntfs_copy_inode(ntfs, ntfs->mft_inode);
+
+    /* cache the data attribute 
+	 *
+	 * This will likely be done already by proc_attrseq, but this
+	 * should be quick
+	 */
+    ntfs->mft_data = fs_data_lookup(ntfs->mft_inode->attr, NTFS_ATYPE_DATA, 0);
+    if (!ntfs->mft_data)
+        error ("Data Attribute not found in $MFT");
+
 
 	/* Get the inode count based on the table size */
 	fs->inum_count = ntfs->mft_data->size / ntfs->mft_rsize_b;
 	fs->last_inum = fs->inum_count - 1;
 
+	/* reset the flag that we are no longer loading $MFT */
+	loading_the_MFT = 0;
+
+	/* load the version of the file system */
+	ntfs_load_ver(ntfs);
 
 	/* load the data block bitmap data run into ntfs_info */
 	ntfs_load_bmap(ntfs);
