@@ -22,6 +22,7 @@ ntfs_attrname_lookup(FS_INFO *, u_int16_t, char *, int);
 /* location of the Root Directory inode */
 #define NTFS_ROOTINO	NTFS_MFT_ROOT	
 #define NTFS_FIRSTINO	0	/* location of the $Mft Record */
+#define NTFS_LAST_DEFAULT_INO	16	/* A guess for right now */
 
 #define NTFS_NDADDR			0
 #define NTFS_NIADDR			0
@@ -58,21 +59,19 @@ typedef struct {
 	u_int8_t	file_ref[6];		// 32 
 	u_int8_t	file_seq[2];		// 38 
 	u_int8_t	max_attr[2];	// 40 max attr id + 1
-	union {
-		struct {
-			u_int8_t	upd_val[2];	// 42
-			u_int8_t	upd_seq;	// 44 array of size 2*(upd_cnt-1) w/orig vals
-		} upd_30;
-		struct {
-			u_int8_t	f1[2];			// 42
-			u_int8_t	mftnum[4];		// 44
-			u_int8_t	upd_val[2];		// 48
-			u_int8_t	upd_seq;		// 50 array of size 2*(upd_cnt-1) w/orig vals
-		} upd_31;
-	} upd;
 } ntfs_mft;
 
+/* Update sequence structure.  This is located at upd_off from the
+ * begining of the original structure
+ */
+typedef struct {
+	u_int8_t	upd_val[2];	// what they should be 
+	u_int8_t	upd_seq;	// array of size 2*(upd_cnt-1) w/orig vals
+} ntfs_upd;
+
+
 #define NTFS_MFT_MAGIC	0x454c4946
+#define NTFS_MFT_MAGIC_BAAD	0x44414142
 
 /* flags for the flags value */
 #define NTFS_MFT_INUSE	0x0001
